"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CSS_CACHE = void 0;
const fs = require("fs");
const url = require("url");
const path = require("path");
const fetch = require("sync-fetch");
const isBrowser = !fs || !url || !path;
const createCache = ({ limit = 100 } = {}) => {
    let cache = {};
    let keys = [];
    return {
        get: (key) => cache[key],
        set: (key, value) => {
            keys.push(key);
            if (keys.length > limit) {
                delete cache[keys.shift()];
            }
            cache[key] = value;
        },
        reset: () => {
            cache = {};
            keys = [];
        },
        length: () => keys.length,
    };
};
exports.CSS_CACHE = createCache({ limit: 30 });
const getAbsoluteLocalPath = (src) => {
    if (isBrowser) {
        // throw new Error('Cannot check local paths in client-side environment');
        return undefined;
    }
    const { protocol, auth, host, port, hostname, path: pathname, } = url.parse(src);
    const absolutePath = path.resolve(pathname);
    if ((protocol && protocol !== 'file:') || auth || host || port || hostname) {
        return undefined;
    }
    return absolutePath;
};
const fetchLocalFile = (src) => {
    if (isBrowser) {
        throw new Error('Cannot fetch local file in this environment');
    }
    const absolutePath = getAbsoluteLocalPath(src);
    if (!absolutePath) {
        throw new Error(`Cannot fetch non-local path: ${src}`);
    }
    return fs.readFileSync(absolutePath, { encoding: 'utf8', flag: 'r' });
};
const fetchRemoteFile = (uri, options) => {
    const response = fetch(uri, options);
    return response.text();
};
const resolveImageFromUrl = (src, crossOrigin = 'anonymous') => {
    return !isBrowser && getAbsoluteLocalPath(src)
        ? fetchLocalFile(src)
        : fetchRemoteFile(src, {
            method: 'GET',
        });
};
const resolveCssFile = (src, crossOrigin = 'anonymous', cache = true) => {
    let image;
    if (cache && exports.CSS_CACHE.get(src)) {
        return exports.CSS_CACHE.get(src);
    }
    else {
        image = resolveImageFromUrl(src, crossOrigin);
    }
    if (!image) {
        throw new Error('Cannot resolve image');
    }
    if (cache) {
        exports.CSS_CACHE.set(src, image);
    }
    return image;
};
exports.default = resolveCssFile;
//# sourceMappingURL=resolveCssFile.js.map